/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package pkg3dtestapplication;

import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.ArrayList;
import javafx.collections.ObservableFloatArray;
import javafx.geometry.Point3D;
import javafx.scene.shape.Mesh;
import javafx.scene.shape.ObservableFaceArray;
import javafx.scene.shape.TriangleMesh;
import javafx.scene.transform.Transform;

public class MeshUtils {

    public static void mesh2STL(String fileName, Mesh mesh) throws IOException {

        if (!(mesh instanceof TriangleMesh)) {
            return;
        }
        // Get faces
        ObservableFaceArray faces = ((TriangleMesh) mesh).getFaces();
        int[] f = new int[faces.size()];
        faces.toArray(f);

        // Get vertices
        ObservableFloatArray points = ((TriangleMesh) mesh).getPoints();
        float[] p = new float[points.size()];
        points.toArray(p);

        StringBuilder sb = new StringBuilder();
        sb.append("solid meshFX\n");

        // convert faces to polygons
        for (int i = 0; i < faces.size() / 6; i++) {
            int i0 = f[6 * i], i1 = f[6 * i + 2], i2 = f[6 * i + 4];
            Point3D pA = new Point3D(p[3 * i0], p[3 * i0 + 1], p[3 * i0 + 2]);
            Point3D pB = new Point3D(p[3 * i1], p[3 * i1 + 1], p[3 * i1 + 2]);
            Point3D pC = new Point3D(p[3 * i2], p[3 * i2 + 1], p[3 * i2 + 2]);
            Point3D pN = pB.subtract(pA).crossProduct(pC.subtract(pA)).normalize();

            sb.append("  facet normal ").append(pN.getX()).append(" ").append(pN.getY()).append(" ").append(pN.getZ()).append("\n");
            sb.append("    outer loop\n");
            sb.append("      vertex ").append(pA.getX()).append(" ").append(pA.getY()).append(" ").append(pA.getZ()).append("\n");
            sb.append("      vertex ").append(pB.getX()).append(" ").append(pB.getY()).append(" ").append(pB.getZ()).append("\n");
            sb.append("      vertex ").append(pC.getX()).append(" ").append(pC.getY()).append(" ").append(pC.getZ()).append("\n");
            sb.append("    endloop\n");
            sb.append("  endfacet\n");
        }

        sb.append("endsolid meshFX\n");

        // write file
        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(fileName), Charset.forName("UTF-8"),
                StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING)) {
            writer.write(sb.toString());
        }
    }

    public static void saveScene(ArrayList<BoxModelObject> objects) throws IOException {

        String fileName = "file" + System.currentTimeMillis()+".fxml";

        StringBuilder sb = new StringBuilder();
        sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
        
        sb.append("<?import javafx.geometry.Point3D?>\n" +
"<?import javafx.scene.Group?>\n" +
"<?import javafx.scene.paint.Color?>\n" +
"<?import javafx.scene.paint.PhongMaterial?>\n" +
"<?import javafx.scene.shape.MeshView?>\n" +
"<?import javafx.scene.shape.TriangleMesh?>\n" +
"<?import javafx.scene.transform.Affine?>\n" +
"<?import javafx.scene.transform.Rotate?>\n");

        
        
        sb.append("<Group>\n");
        sb.append("<children>\n");
        
        for (BoxModelObject o : objects) {
            
            Mesh mesh = o.getMesh();
            // Get faces
            ObservableFaceArray faces = ((TriangleMesh) mesh).getFaces();
            int[] f = new int[faces.size()];
            faces.toArray(f);
            
            int fsmg = faces.size()/6;

            // Get vertices
            ObservableFloatArray points = ((TriangleMesh) mesh).getPoints();
            float[] p = new float[points.size()];
            points.toArray(p);
            sb.append("<MeshView>\n");
            
            //Transforms
            sb.append("<transforms>\n");
            
            for(Transform t : o.getTransforms()){
                String affine = "<Affine mxx=\""+t.getMxx()+"\" mxy=\""+t.getMxy()+"\" mxz=\""+t.getMxz()+"\" tx=\""+t.getTx()+"\" myx=\""+t.getMyx()+"\" myy=\""+t.getMyy()+"\" myz=\""+t.getMyz()+"\" ty=\""+t.getTy()+"\" mzx=\""+t.getMzx()+"\" mzy=\""+t.getMzy()+"\" mzz=\""+t.getMzz()+"\" tz=\""+t.getTz()+"\"/>\n";
                sb.append(affine); 
            }
            
            sb.append("</transforms>\n");
            
            
            sb.append("<material>\n");
            sb.append("<PhongMaterial specularPower=\"32.0\">\n");
            sb.append("<diffuseColor>\n");
            String color = "<Color red=\""+o.getColor().getDiffuseColor().getRed()+"\" green=\""+o.getColor().getDiffuseColor().getGreen()+"\" blue=\""+o.getColor().getDiffuseColor().getBlue()+"\" opacity=\"1.0\"/>\n";
            sb.append(color);
            sb.append("</diffuseColor>\n");
            sb.append("</PhongMaterial>\n");
            sb.append("</material>\n");
            
            
            sb.append("<mesh>\n");
            sb.append("<TriangleMesh>\n");
            sb.append("<points>");
            for(float pp : p){
                sb.append(pp+" ");
            }
            sb.append("</points>\n");
            sb.append("<texCoords>0.0 0.0</texCoords>\n");
            sb.append("<faces>");
            for(int ff : f){
                sb.append(ff+" ");
            }
            sb.append("</faces>\n");
            
            sb.append("<faceSmoothingGroups>");
            
            for(int i = 0; i<fsmg; i++){
                sb.append("0 ");
            }
            sb.append("</faceSmoothingGroups>\n");
            
            sb.append("</TriangleMesh>\n");
            sb.append("</mesh>\n");
            sb.append("</MeshView>\n");
        }

        sb.append("</children>\n");
        sb.append("</Group>\n");

        // write file
        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(fileName), Charset.forName("UTF-8"),
                StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING)) {
            writer.write(sb.toString());
        }

    }
}
