/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package pkg3dtestapplication;

import java.io.IOException;
import java.util.ArrayList;
import javafx.event.EventType;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Point2D;
import javafx.geometry.Point3D;
import javafx.scene.AccessibleRole;
import javafx.scene.AmbientLight;
import javafx.scene.Group;
import javafx.scene.PointLight;
import javafx.scene.input.PickResult;
import javafx.scene.input.TouchEvent;
import javafx.scene.paint.Color;
import javafx.scene.shape.Box;
import javafx.scene.shape.DrawMode;
import javafx.scene.shape.Mesh;
import javafx.scene.shape.MeshView;
import javafx.scene.shape.TriangleMesh;
import org.fxyz.shapes.Pyramid;
import org.fxyz.shapes.Torus;
import org.fxyz.shapes.composites.SurfacePlot;
import org.fxyz.tools.CubeViewer;

/**
 *
 * @author henri
 */
public class World3D extends Xform {

    private float width, depth;
    
    private TriangleMesh mesh;
    private Group model;

    private World3D(Builder builder) {
        super();
        this.width = builder.width;
        this.depth = builder.depth;
        this.model = loadModel();
        getHorizontalEdgeLoop(0.3f,0.2f);
        this.getChildren().addAll(model);

        AmbientLight ambient = new AmbientLight();
        ambient.setColor(Color.rgb(50, 50, 50, 0.6));
        this.getChildren().add(ambient);

        PointLight light = new PointLight();
        light.setColor(Color.rgb(50, 50, 50, 0.6));
        light.setTranslateY(-50);
        this.getChildren().add(light);

    }

    public Point2D getDimension() {
        return new Point2D(width, depth);
    }

    private Group loadModel() {
        try {
            FXMLLoader fxmlLoader = new FXMLLoader();
            fxmlLoader.setLocation(this.getClass().getResource("/resources/cube.fxml"));
            model = fxmlLoader.<Group>load();
            
            MeshView meshView = (MeshView)model.getChildren().get(0);
            meshView.setDrawMode(DrawMode.LINE);
            meshView.setScaleX(2);
            meshView.setScaleY(2);
            meshView.setScaleZ(2);           
            mesh = (TriangleMesh)meshView.getMesh();
            
        } catch (IOException e) {
            // exception handling
        }
        return model;
    }
    
    public void getHorizontalEdgeLoop(float height,float range){
    
        int numPoints = mesh.getPoints().size()/3;
    
        for(int i = 0; i < numPoints; i++){
            float x = mesh.getPoints().get(i*3);
            float y = mesh.getPoints().get(i*3+1);
            float z = mesh.getPoints().get(i*3+2);
            System.out.println(x+", "+y+", "+z);
            
            if(isInRange(mesh.getPoints().get(i*3+2),height,range)){
                //points.add(new Point3D(x,y,z));
                float t = mesh.getPoints().get(i*3)*5;
                mesh.getPoints().set(i*3, t);  
            }
        }
        

    }
    
    private boolean isInRange(float p,float height, float range){
        boolean isInRange = false;
        
        float delta = Math.abs(height - p);
        
        if(delta<=range){
            isInRange = true;
        }
        
        return isInRange;
    
    }


    public static class Builder {

        private final float width;
        private final float depth;

        public Builder(float width, float depth) {
            this.width = width;
            this.depth = depth;
        }

        public World3D build() {
            return new World3D(this);
        }

    }

}
